"""Define actions library for thruk-agent charm."""

import grp
import hashlib
import os
import pwd
import stat
import subprocess
import sys


from charmhelpers.core import hookenv
from charmhelpers.fetch import (
    add_source,
    apt_install,
    apt_update,
    filter_installed_packages,
)


PACKAGE_LIST = ["thruk", "pwgen", "apache2-utils"]


def log_start(service_name):
    """Log that the application has started in the unit log."""
    hookenv.log("thruk-agent starting")


def fixpath(path):
    """Update permissions of 'path' and heirarchy above to allow thruk access."""
    if os.path.isdir(path):
        st = os.stat(path)
        os.chmod(path, st.st_mode | stat.S_IXOTH)
    if path != "/":
        fixpath(os.path.split(path)[0])


def pwgen():
    """Generate random password to assign to thrukadmin."""
    return str(subprocess.check_output(["pwgen", "-s", "16"])).strip()


def fix_livestatus_perms(service_name):
    """Resolve issues with livestatus socket permissions from nagios."""
    livestatus_path = hookenv.config("livestatus_path")
    if os.path.exists(livestatus_path):
        livestatus_dir = os.path.dirname(livestatus_path)
        uid = pwd.getpwnam("nagios").pw_uid
        gid = grp.getgrnam("www-data").gr_gid
        os.chown(livestatus_path, uid, gid)
        os.chown(livestatus_dir, uid, gid)
        os.chmod(livestatus_path, 0o770)
        os.chmod(livestatus_dir, 0o2771)
        fixpath(livestatus_path)
    else:
        hookenv.log("ERROR: livestatus socket doesn't exist")
        sys.exit(1)


def thruk_set_password(service_name):
    """Configure the htaccess password for thrukadmin in unit's local webserver."""
    passwd_file = "/var/lib/thruk/thrukadmin.passwd"
    if not os.path.exists(passwd_file):
        password = pwgen()
        with open(passwd_file, "w") as pfile:
            pfile.write(password)
            os.chmod(pfile.name, 0o600)

        ret = subprocess.call(
            ["/usr/bin/htpasswd", "-b", "/etc/thruk/htpasswd", "thrukadmin", password]
        )
        if not ret:
            hookenv.log("WARNING: thruk htpassword reset failed!")
    else:
        with open(passwd_file) as pfile:
            password = pfile.read().strip()


def notify_thrukmaster_relation(service_name):
    """Provide relation data to thruk-master."""
    # rel_data = thruk_helpers.ThrukAgentRelation()
    thruk_data = {}

    # Need to check that we actually have a thruk-agent relation
    # before trying to pull out the data
    host = hookenv.unit_get("private-address")
    url = "http://{}/".format(host)
    thruk_data["url"] = url

    keypath = "/var/lib/thruk/secret.key"
    with open(keypath) as keyfile:
        key = keyfile.read()
        thruk_key = key.rstrip("\n")
    thruk_data["thruk_key"] = thruk_key

    m = hashlib.md5()
    m.update(hookenv.config("nagios_context"))
    thruk_data["thruk_id"] = m.hexdigest()
    thruk_data["nagios_context"] = hookenv.config("nagios_context")

    for rel_id in hookenv.relation_ids("thruk-agent"):
        hookenv.relation_set(relation_id=rel_id, relation_settings=thruk_data)


def install_cookie_into_apache2():
    """Activate cookie in existing default virtual hosts."""
    thruk_cookie = "/etc/apache2/vhost.d/thruk_cookie_auth.include"
    try:
        os.makedirs("/etc/apache2/vhost.d/")
    except OSError:
        pass

    try:
        os.remove(thruk_cookie)
    except OSError:
        pass

    os.symlink("/usr/share/thruk/thruk_cookie_auth.include", thruk_cookie)


def update_ppa(service_name):
    """Update PPA for installation of thruk-agent binaries."""
    config = hookenv.config()
    prev_source = config.previous("source")

    if prev_source is not None and config.changed("source"):
        subprocess.check_call(
            ["add-apt-repository", "--yes", "--remove", prev_source],
            env=hookenv.env_proxy_settings(["https"]),
        )
    if config.changed("source") or config.changed("key"):
        add_source(config.get("source"), config.get("key", None))
        apt_update(fatal=True)

    pkgs_to_install = filter_installed_packages(PACKAGE_LIST)
    if pkgs_to_install:
        apt_install(packages=pkgs_to_install, fatal=True)
    install_cookie_into_apache2()


def update_status(service_name):
    """Set unit status."""
    hookenv.status_set("active", "ready")
