#!/usr/bin/python3.6
"""Define functional tests for thruk-agent charm."""

import os

import pytest

import requests

pytestmark = pytest.mark.asyncio

CHARM_BUILD_DIR = os.getenv("CHARM_BUILD_DIR", "..").rstrip("/")

SERIES = [
    "trusty",
    "xenial",
    "bionic",
]


############
# FIXTURES #
############


@pytest.fixture(scope="module", params=SERIES)
def series(request):
    """Return ubuntu version (i.e. xenial) in use in the test."""
    return request.param


@pytest.fixture(scope="module")
async def deploy_app(model, series):
    """Return application of the charm under test."""
    app_name = "thruk-agent-{}".format(series)
    nagios_name = "nagios-{}".format(series)

    """Deploy the thruk_agent app as a subordinate of nagios."""
    await model.deploy(
        "cs:nagios",
        application_name=nagios_name,
        series=series,
        channel="stable",
        config={"enable_livestatus": True},
    )
    thruk_agent_app = await model.deploy(
        os.path.join(CHARM_BUILD_DIR, "thruk-agent"),
        application_name=app_name,
        series=series,
        num_units=0,
        config={},
    )
    await model.add_relation(
        "{}:juju-info".format(nagios_name), "{}:general-info".format(app_name)
    )

    await model.block_until(lambda: thruk_agent_app.status == "active")
    # no need to cleanup since the model will be be torn down at the end of the
    # testing

    yield thruk_agent_app


@pytest.fixture(scope="module")
async def unit(deploy_app):
    """Return the thruk_agent unit we've deployed."""
    return deploy_app.units[0]


#########
# TESTS #
#########


async def test_status(deploy_app):
    """Check that the app is in active state."""
    assert deploy_app.status == "active"


async def test_http(model, file_contents, unit, series):
    """Check the thruk http interface."""
    principalname = "nagios-{}".format(series)
    thrukpwd = await file_contents("/var/lib/thruk/thrukadmin.passwd", unit)
    nagios_unit = model.applications[principalname].units[0]
    host_url = "http://%s/thruk/" % nagios_unit.public_address
    auth = "thrukadmin", thrukpwd
    requests.get(host_url, auth=auth)


async def test_remove_relation(model, run_command, series):
    """Check that the relation is removed."""
    app_name = "thruk-agent-{}".format(series)
    principalname = "nagios-{}".format(series)
    app = model.applications[app_name]
    await app.remove_relation(
        "{}:general-info".format(app_name), "{}:juju-info".format(principalname)
    )
    await model.block_until(lambda: not app.relations)
    principal = model.applications[principalname].units[0]
    res = await run_command("test -f /etc/sudo.conf || echo gone", target=principal)
    assert res["Stdout"].strip() == "gone"
    await model.add_relation(
        "{}:juju-info".format(principalname), "{}:general-info".format(app_name)
    )
    await model.block_until(lambda: app.relations)
